local ADDON, Package = ...

local Module = {}
local Registry = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------
local startcase = Package.String.startcase
local merge = Package.Enum.merge

--------------------------------------------------------------------------------
-- Localization
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local PROXY = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.load_storage()
  local default_schema = {
    version = 2.0,
    use_item_quality_color_border = true,
    use_arrow_keys_in_chat = true,
    increase_chat_history = true,
    unclamp_chat_frame = true,
    use_sticky_raid_warning_in_chat = true,
    use_class_color_in_chat = true,
    use_max_distance = true,
    use_max_zoom_factor = true,
    posture_after_action_attempted = true,
    repair_when_interact_merchant = true,
    sell_junk_when_interact_merchant = true,
    rework_time_alert = true,
    search_profession = true,
    indicate_accurate_buff_time = true,
    indicate_item_id_and_level_on_tooltip = true,
    indicate_item_vendor_price_on_tooltip = true,
    indicate_target_level_and_class_color_in_trade = true,
    copy_url_in_chat = true,
    log_chat = true,
    prevent_unit_combat_feedback = false,
    prevent_ui_error_message_during_combat = false,
  }

  local chat_log_schema = {
    chat_log = {
      ChatFrame1 = {},
      ChatFrame3 = {},
      ChatFrame4 = {},
      ChatFrame5 = {},
      ChatFrame6 = {},
      ChatFrame7 = {},
      ChatFrame8 = {},
      ChatFrame9 = {},
      ChatFrame10 = {}
    }
  }

  MedusaPreludeStorage = merge(MedusaPreludeStorage or {}, default_schema, false)
  MedusaPreludeStoragePerCharacter = merge(MedusaPreludeStoragePerCharacter or {}, chat_log_schema, false)
end

function Module.run()
  PROXY:RegisterEvent("PLAYER_LOGIN")
  PROXY:RegisterEvent("ADDON_LOADED")
  PROXY:SetScript("OnEvent", Module.on_event)
end

function Module.on_event(self, event, ...)
  Registry[event](self, ...)
end

--------------------------------------------------------------------------------
-- Registry
--------------------------------------------------------------------------------
function Registry.PLAYER_LOGIN()
  local keys = {
    use_item_quality_color_border = "UseItemQualityColorBorder",
    use_arrow_keys_in_chat = "UseArrowKeysInChat",
    increase_chat_history = "IncreaseChatHistory",
    unclamp_chat_frame = "UnclampChatFrame",
    use_sticky_raid_warning_in_chat = "UseStickyRaidWarningInChat",
    use_class_color_in_chat = "UseClassColorInChat",
    use_max_distance = "UseMaxDistance",
    use_max_zoom_factor = "UseMaxZoomFactor",
    posture_after_action_attempted = "PostureAfterActionAttempted",
    repair_when_interact_merchant = "RepairWhenInteractMerchant",
    sell_junk_when_interact_merchant = "SellJunkWhenInteractMerchant",
    rework_time_alert = "ReworkTimeAlert",
    search_profession = "SearchProfession",
    indicate_accurate_buff_time = "IndicateAccurateBuffTime",
    indicate_item_id_and_level_on_tooltip = "IndicateItemIDAndLevelOnTooltip",
    indicate_item_vendor_price_on_tooltip = "IndicateItemVendorPriceOnTooltip",
    indicate_target_level_and_class_color_in_trade = "IndicateTargetLevelAndClassColorInTrade",
    copy_url_in_chat = "CopyURLInChat",
    log_chat = "LogChat",
    prevent_unit_combat_feedback = "PreventUnitCombatFeedback",
    prevent_ui_error_message_during_combat = "PreventUIErrorMessageDuringCombat"
  }

  for option, value in pairs(MedusaPreludeStorage) do
    local name = keys[option]

    if Package[name] then
      if value then
        Package[name].enable()
      else
        Package[name].disable()
      end
    end
  end
end

function Registry.ADDON_LOADED(self, ...)
  if ADDON == ... then
    Module.load_storage()

    local addon = WrapTextInColorCode(..., "ff91d7f2")
    local version = WrapTextInColorCode("v" .. GetAddOnMetadata(..., "Version"), "ff91d7f2")
    local slashes = WrapTextInColorCode(SLASH_MedusaPrelude1 .. " " .. SLASH_MedusaPrelude2, "ff72f2e5")

    DEFAULT_CHAT_FRAME:AddMessage(addon .. " " .. version .. " " .. slashes )
  end
end

--------------------------------------------------------------------------------
-- Slash Command
--------------------------------------------------------------------------------
SLASH_MedusaPrelude1 = "/medusa_prelude"
SLASH_MedusaPrelude2 = "/mp"
SlashCmdList[ADDON] = function ()
  if not MedusaPreludeConfig then
    Package.Config.run()

    table.insert(UISpecialFrames, MedusaPreludeConfig:GetName())
  end

  MedusaPreludeConfig:SetShown(not MedusaPreludeConfig:IsShown())
end

--------------------------------------------------------------------------------
-- Entry Point
--------------------------------------------------------------------------------
Module.run()
