local ADDON, Package = ...

local Module = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local CHAT_FRAMES = {
  ChatFrame1 = ChatFrame1,
  -- ChatFrame2 = ChatFrame2,
  ChatFrame3 = ChatFrame3,
  ChatFrame4 = ChatFrame4,
  ChatFrame5 = ChatFrame5,
  ChatFrame6 = ChatFrame6,
  ChatFrame7 = ChatFrame7,
  ChatFrame8 = ChatFrame8,
  ChatFrame9 = ChatFrame9,
  ChatFrame10 = ChatFrame10
}

local URL_DELIMITER = "://"

local URL_MATCH = "^(.*)(https?://[^ ]+)(.*)$"

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------
local function create_link(link_type, link_text)
  return string.format("|H%s:|h[%s]|h", link_type, link_text)
end

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local is_configured = false
local preserve = {}

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.SetItemRef(link, text, button, chat_frame)
  if link:sub(1, 4) == "item" and text:find(URL_DELIMITER) then
    StaticPopup_Show(ADDON .. "Clipboard", nil, nil, text:match("%[(.+)%]"))
  else
    preserve.SetItemRef(link, text, button, chat_frame)
  end
end

function Module.ChatFrame_AddMessage(self, message, ...)
  if message and message:find(URL_DELIMITER) then
    local pre, url, post = message:match(URL_MATCH)

    if pre and url and post then
      message = pre .. create_link("item", url) .. post
    end
  end

  preserve[self:GetName()](self, message, ...)
end

function Module.configure(reset)
  if reset then
    SetItemRef = preserve.SetItemRef

    preserve.SetItemRef = nil
  else
    preserve.SetItemRef = SetItemRef

    SetItemRef = Module.SetItemRef
  end

  local frames = CHAT_FRAMES

  for name, frame in pairs(frames) do
    if frame.AddMessage then
      if reset then
        frame.AddMessage = preserve[name]

        preserve[name] = nil
      else
        preserve[name] = frame.AddMessage

        frame.AddMessage = Module.ChatFrame_AddMessage
      end
    end
  end
end

function Module.enable()
  if not is_configured then
    is_configured = true

    Module.configure(false)
  end
end

function Module.disable()
  if is_configured then
    is_configured = false

    Module.configure(true)
  end
end

--------------------------------------------------------------------------------
-- Static Popup
--------------------------------------------------------------------------------
StaticPopupDialogs[ADDON .. "Clipboard"] = {
  button1 = CLOSE,
  whileDead = true,
  hideOnEscape = true,
  hasEditBox = true,
  timeout = 0,
  text = "",
  OnShow = function (self)
    self.editBox:SetText(self.data)
    self.editBox:HighlightText()
    self.editBox:SetCursorPosition(0)

    if not self.editBox:GetScript("OnMouseUp") then
      self.editBox:SetScript("OnMouseUp", function (self) self:HighlightText() end)
    end
  end,
  OnHide = function (self)
    self.data = nil
  end,
  EditBoxOnEscapePressed = function(self)
    self:GetParent():Hide()
    ClearCursor()
  end,
  EditBoxOnTextChanged = function (self)
    self:SetText(self:GetParent().data)
    self:HighlightText()
  end
}

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.CopyURLInChat = Module