local ADDON, Package = ...

local Module = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local TOOLTIP_TEXT_RIGHT_REFS = {
  GameTooltip = GameTooltipTextRight1,
  ShoppingTooltip1 = ShoppingTooltip1TextRight2,
  ShoppingTooltip2 = ShoppingTooltip2TextRight2,
  ItemRefTooltip = ItemRefTooltipTextRight1
}

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local is_hooked = false

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.handle(self)
  local _, item_link = self:GetItem()

  if item_link then
    local item_id = GetItemInfoFromHyperlink(item_link)
    local _, _, _, item_level = GetItemInfo(item_link)

    if self:IsShown() and item_id and item_level then
      local text = TOOLTIP_TEXT_RIGHT_REFS[self:GetName()]
      local item_id_text = GRAY_FONT_COLOR:WrapTextInColorCode(item_id)
      local item_level_text = NORMAL_FONT_COLOR:WrapTextInColorCode(ITEM_LEVEL:format(item_level))

      if text == ItemRefTooltipTextRight1 then
        text:SetFormattedText("%s %s", item_id_text, item_level_text)
      else
        text:SetFormattedText(IsAltKeyDown() and item_id_text or item_level_text)
      end

      text:Show()
    end
  end

  self:Show()
end

function Module.OnTooltipSetItem(...) Module.OnTooltipSetItem_Stub(...) end
function Module.OnTooltipSetItem_Stub() end
function Module.OnTooltipSetItem_Enable(self) Module.handle(self) end
function Module.OnTooltipSetItem_Disable() end

function Module.SetHyperlink(...) Module.SetHyperlink_Stub(...) end
function Module.SetHyperlink_Stub() end
function Module.SetHyperlink_Enable(self) Module.handle(self) end
function Module.SetHyperlink_Disable() end

function Module.enable()
  Module.OnTooltipSetItem_Stub = Module.OnTooltipSetItem_Enable
  Module.SetHyperlink_Stub = Module.SetHyperlink_Enable

  if not is_hooked then
    is_hooked = true

    GameTooltip:HookScript("OnTooltipSetItem", Module.OnTooltipSetItem)
    ItemRefTooltip:HookScript("OnTooltipSetItem", Module.OnTooltipSetItem)
    ShoppingTooltip1:HookScript("OnTooltipSetItem", Module.OnTooltipSetItem)
    ShoppingTooltip2:HookScript("OnTooltipSetItem", Module.OnTooltipSetItem)

    hooksecurefunc(GameTooltip, "SetHyperlink", Module.SetHyperlink)
    hooksecurefunc(ItemRefTooltip, "SetHyperlink", Module.SetHyperlink)
  end
end

function Module.disable()
  Module.OnTooltipSetItem_Stub = Module.OnTooltipSetItem_Disable
  Module.SetHyperlink_Stub = Module.SetHyperlink_Disable
end

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.IndicateItemIDAndLevelOnTooltip = Module
