local ADDON, Package = ...

local Module = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local CHAT_FRAMES = {
  ChatFrame1 = ChatFrame1,
  -- ChatFrame2 = ChatFrame2,
  ChatFrame3 = ChatFrame3,
  ChatFrame4 = ChatFrame4,
  ChatFrame5 = ChatFrame5,
  ChatFrame6 = ChatFrame6,
  ChatFrame7 = ChatFrame7,
  ChatFrame8 = ChatFrame8,
  ChatFrame9 = ChatFrame9,
  ChatFrame10 = ChatFrame10
}

local MAX_CHAT_LOG = 64

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------
local function darken_color_markup(hex)
  return string.format("|cff%06x", bit.rshift(bit.band(tonumber(hex, 16), 0xfefefe), 1))
end

local function darken_color_message(message, r, g, b, ...)
  return {(message or ""):gsub("|c%x%x(%x%x%x%x%x%x)", darken_color_markup), (r or 1) * 0.82, (g or 1) * 0.82, (b or 1) * 0.82, ...}
end

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local is_configured = false
local preserve = {}

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.ChatFrame_AddMessage(self, message, ...)
  preserve[self:GetName()](self, message, ...)

  local storage = MedusaPreludeStoragePerCharacter.chat_log[self:GetName()]

  if message:find("BNplayer") then
    local account_id = message:match(" |HBNplayer:[^:]+:([0-9]+).+|h ")
    local _, _, battle_tag = BNGetFriendInfoByID(account_id or 0)

    if battle_tag then
      message = message:gsub("^([^|]+)|HBNplayer:[^%[]+%[[^%]]+%]|h", "%1[" .. battle_tag:gsub("#[0-9]+", "") .. "]")
    end
  end

  table.insert(storage, 1, {message, ...})

  if #storage > MAX_CHAT_LOG then
    table.remove(storage)
  end
end

function Module.configure(reset)
  local frames = CHAT_FRAMES

  for name, frame in pairs(frames) do
    if frame.AddMessage then
      if reset then
        frame.AddMessage = preserve[name]

        preserve[name] = nil
      else
        preserve[name] = frame.AddMessage

        frame.AddMessage = Module.ChatFrame_AddMessage
      end
    end
  end
end

function Module.revert_history()
  for name, messages in pairs(MedusaPreludeStoragePerCharacter.chat_log) do
    for at = #messages, 1, -1 do
      local message = darken_color_message(unpack(messages[at]))

      CHAT_FRAMES[name]:AddMessage(unpack(message))
    end
  end
end

function Module.enable()
  Module.revert_history()

  if not is_configured then
    is_configured = true

    Module.configure(false)
  end
end

function Module.disable()
  if is_configured then
    is_configured = false

    Module.configure(true)
  end
end

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.LogChat = Module