local ADDON, Package = ...

local Module = {}
local Registry = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local PROXY = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------
local function print_money(amount, message, color)
  local text = color:WrapTextInColorCode(message)
  local coin = GetCoinTextureString(amount):gsub(":0", ":-4")

  DEFAULT_CHAT_FRAME:AddMessage(text .. " " .. coin)
end

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local ticker = nil
local items = 0
local price = 0

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.tick(args)
  local pending = 0

  for bag = 0, NUM_BAG_FRAMES do
    for slot = 1, GetContainerNumSlots(bag) do
      local item_id = GetContainerItemID(bag, slot)

      if item_id then
        local _, _, item_rarity, _, _, _, _, _, _, _, item_sell_price = GetItemInfo(item_id)
        local _, item_count, item_locked = GetContainerItemInfo(bag, slot)

        if MerchantFrame:IsShown() and not item_locked and item_rarity == 0 and item_sell_price > 0 then
          UseContainerItem(bag, slot)

          pending = pending + 1

          items = items + 1
          price = price + (item_sell_price * item_count)
        end
      end
    end
  end

  if pending == 0 and not ticker:IsCancelled() then
    if items > 0 and price > 0 then
      print_money(price, ITEM_QUALITY0_DESC .. 'x' .. items, ITEM_QUALITY_COLORS[0].color)
    end

    ticker:Cancel()
  end
end

function Module.enable()
  PROXY:RegisterEvent("MERCHANT_SHOW")
  PROXY:RegisterEvent("MERCHANT_CLOSED")
  PROXY:SetScript("OnEvent", Module.on_event)
end

function Module.disable()
  PROXY:UnregisterEvent("MERCHANT_SHOW")
  PROXY:UnregisterEvent("MERCHANT_CLOSED")
  PROXY:SetScript("OnEvent", nil)
end

function Module.on_event(self, event, ...)
  Registry[event](self, ...)
end

--------------------------------------------------------------------------------
-- Registry
--------------------------------------------------------------------------------
function Registry.MERCHANT_SHOW()
  if not IsShiftKeyDown() then
    if ticker then
      ticker:Cancel()
    end

    items = 0
    price = 0

    ticker = C_Timer.NewTicker(0.2, Module.tick, 20)
  end
end

function Registry.MERCHANT_CLOSED()
  if ticker and not ticker:IsCancelled() then
    if items > 0 and price > 0 then
      print_money(price, ITEM_QUALITY0_DESC .. 'x' .. items, ITEM_QUALITY_COLORS[0].color)
    end

    ticker:Cancel()
  end
end

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.SellJunkWhenInteractMerchant = Module