local ADDON, Package = ...

local Module = {}
local Registry = {}

local PAPER_DOLL_ITEMS = {
  HeadSlot = "CharacterHeadSlot",
  NeckSlot = "CharacterNeckSlot",
  ShoulderSlot = "CharacterShoulderSlot",
  BackSlot = "CharacterBackSlot",
  ChestSlot = "CharacterChestSlot",
  ShirtSlot = "CharacterShirtSlot",
  TabardSlot = "CharacterTabardSlot",
  WristSlot = "CharacterWristSlot",
  HandsSlot = "CharacterHandsSlot",
  WaistSlot = "CharacterWaistSlot",
  LegsSlot = "CharacterLegsSlot",
  FeetSlot = "CharacterFeetSlot",
  Finger0Slot = "CharacterFinger0Slot",
  Finger1Slot = "CharacterFinger1Slot",
  Trinket0Slot = "CharacterTrinket0Slot",
  Trinket1Slot = "CharacterTrinket1Slot",
  MainHandSlot = "CharacterMainHandSlot",
  SecondaryHandSlot = "CharacterSecondaryHandSlot",
  RangedSlot = "CharacterRangedSlot",
  AmmoSlot = "CharacterAmmoSlot"
}

local INSPECT_PAPER_DOLL_ITEMS = {
  HeadSlot = "InspectHeadSlot",
  NeckSlot = "InspectNeckSlot",
  ShoulderSlot = "InspectShoulderSlot",
  BackSlot = "InspectBackSlot",
  ChestSlot = "InspectChestSlot",
  ShirtSlot = "InspectShirtSlot",
  TabardSlot = "InspectTabardSlot",
  WristSlot = "InspectWristSlot",
  HandsSlot = "InspectHandsSlot",
  WaistSlot = "InspectWaistSlot",
  LegsSlot = "InspectLegsSlot",
  FeetSlot = "InspectFeetSlot",
  Finger0Slot = "InspectFinger0Slot",
  Finger1Slot = "InspectFinger1Slot",
  Trinket0Slot = "InspectTrinket0Slot",
  Trinket1Slot = "InspectTrinket1Slot",
  MainHandSlot = "InspectMainHandSlot",
  SecondaryHandSlot = "InspectSecondaryHandSlot",
  RangedSlot = "InspectRangedSlot"
}

local agent = CreateFrame("Frame")

function create_border(slot_name, frame)
  if frame and not frame.ItemQualityBorder then
    local id = GetInventorySlotInfo(slot_name)
    local item_quality_color = ITEM_QUALITY_COLORS[item_quality]
    local border = frame:CreateTexture(nil, "OVERLAY", nil, -1)

    if name == "AmmoSlot" then
      border:SetSize(47, 47)
    else
      border:SetSize(65, 65)
    end

    border:SetBlendMode("ADD")
    border:SetTexture(130739)
    border:SetPoint("CENTER")
    border:Hide()

    frame.ItemQualityBorder = border
  end
end

function update_border(slot_name, frame, unit_id)
  local id = GetInventorySlotInfo(slot_name)
  local item_quality = GetInventoryItemQuality(unit_id, id)

  if id and item_quality and frame and frame.ItemQualityBorder then
    local item_quality_color = ITEM_QUALITY_COLORS[item_quality]

    frame.ItemQualityBorder:SetVertexColor(item_quality_color.r, item_quality_color.g, item_quality_color.b)
    frame.ItemQualityBorder:Show()
  else
    frame.ItemQualityBorder:Hide()
  end
end

function create_border_in_paper_doll_items()
  for slot_name, frame_name in pairs(PAPER_DOLL_ITEMS) do
    create_border(slot_name, _G[frame_name])
  end
end

function update_border_in_paper_doll_items()
  for slot_name, frame_name in pairs(PAPER_DOLL_ITEMS) do
    update_border(slot_name, _G[frame_name], "player")
  end
end

function create_border_in_inspect_paper_doll_items()
  for slot_name, frame_name in pairs(INSPECT_PAPER_DOLL_ITEMS) do
    create_border(slot_name, _G[frame_name])
  end
end

function update_border_in_inspect_paper_doll_items(x)
  for slot_name, frame_name in pairs(INSPECT_PAPER_DOLL_ITEMS) do
    update_border(slot_name, _G[frame_name], x)
  end
end

function Module.enable()
  agent:SetScript("OnEvent", Module.on_event)
  agent:RegisterEvent("UPDATE_INVENTORY_DURABILITY")
  agent:RegisterEvent("INSPECT_READY")

  PaperDollFrame:HookScript("OnShow", create_border_in_paper_doll_items)
  PaperDollFrame:HookScript("OnShow", update_border_in_paper_doll_items)
end

function Module.disable()
  agent:SetScript("OnEvent", nil)
  agent:UnregisterEvent("UPDATE_INVENTORY_DURABILITY")
  agent:UnregisterEvent("INSPECT_READY")
end

function Module.on_event(self, event, ...)
  Registry[event](self, ...)
end

function Registry.UPDATE_INVENTORY_DURABILITY()
  if PaperDollFrame:IsVisible() then
    create_border_in_paper_doll_items()
    update_border_in_paper_doll_items()
  end
end

function Registry.INSPECT_READY(inspectee_guid)
  create_border_in_inspect_paper_doll_items()
  update_border_in_inspect_paper_doll_items("target")
end

Package.UseItemQualityColorBorder = Module